@@ -5,7 +5,7 @@ test_that("Assess mean_median_surv_barplot_f", {
 
   # Run the function with the selected sample
   p1 <- mean_median_surv_barplot_f(surv_mean_med_data = mean_median_survival_time_data,
-                                   selected_sample = "TCGA-TQ-A8XE-01",
+                                   selected_sample = "TCGA-TQ-A7RQ-01",
 				   font_size = 12, font_color = "darkblue")
 
   # Check that the output is a list
